package JSON::Karabiner::Manipulator::Conditions::Frontmost_application_unless ;
$JSON::Karabiner::Manipulator::Conditions::Frontmost_application_unless::VERSION = '0.004';
use strict;
use warnings;
use JSON;
use Carp;
use parent 'JSON::Karabiner::Manipulator::Conditions::Frontmost_application_if';


# ABSTRACT: definition for Frontmost_application_unless condition

1;

__END__

=pod

=head1 NAME

JSON::Karabiner::Manipulator::Conditions::Frontmost_application_unless - definition for Frontmost_application_unless condition

=head1 SYNOPSIS

  use JSON::Karabiner;

=head1 DESCRIPTION

=head3 method1()

=head3 method2()

=head1 VERSION

version 0.004

=head1 AUTHOR

Steve Dondley <s@dondley.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2020 by Steve Dondley.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
